@@ -36,17 +36,16 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 
-import javax.xml.bind.DatatypeConverter;
-
 /**
  * Conversion between JDK classes and a {@code String}.
  */
-enum JDKStringConverter implements StringConverter<Object> {
+enum JDKStringConverter implements TypedStringConverter<Object> {
 
     /**
      * String converter.
      */
     STRING(String.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return str;
         }
@@ -55,6 +54,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * CharSequence converter.
      */
     CHAR_SEQUENCE(CharSequence.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return str;
         }
@@ -63,6 +63,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * StringBuffer converter.
      */
     STRING_BUFFER(StringBuffer.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new StringBuffer(str);
         }
@@ -71,6 +72,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * StringBuilder converter.
      */
     STRING_BUILDER(StringBuilder.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new StringBuilder(str);
         }
@@ -79,6 +81,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Long converter.
      */
     LONG(Long.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Long(str);
         }
@@ -88,6 +91,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Integer converter.
      */
     INTEGER(Integer.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Integer(str);
         }
@@ -97,6 +101,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Short converter.
      */
     SHORT (Short.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Short(str);
         }
@@ -106,6 +111,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Byte converter.
      */
     BYTE(Byte.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Byte(str);
         }
@@ -116,16 +122,18 @@ enum JDKStringConverter implements StringConverter<Object> {
     BYTE_ARRAY(byte[].class) {
         @Override
         public String convertToString(Object object) {
-            return DatatypeConverter.printBase64Binary((byte[]) object);
+            return printBase64Binary((byte[]) object);
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
-            return DatatypeConverter.parseBase64Binary(str);
+            return parseBase64Binary(str);
         }
     },
     /**
      * Character converter.
      */
     CHARACTER(Character.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             if (str.length() != 1) {
                 throw new IllegalArgumentException("Character value must be a string length 1");
@@ -141,6 +149,7 @@ enum JDKStringConverter implements StringConverter<Object> {
         public String convertToString(Object object) {
             return new String((char[]) object);
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return str.toCharArray();
         }
@@ -149,6 +158,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Boolean converter.
      */
     BOOLEAN(Boolean.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             if ("true".equalsIgnoreCase(str)) {
                 return Boolean.TRUE;
@@ -163,6 +173,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Double converter.
      */
     DOUBLE(Double.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Double(str);
         }
@@ -171,6 +182,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Float converter.
      */
     FLOAT(Float.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new Float(str);
         }
@@ -179,6 +191,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * BigInteger converter.
      */
     BIG_INTEGER(BigInteger.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new BigInteger(str);
         }
@@ -187,6 +200,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * BigDecimal converter.
      */
     BIG_DECIMAL(BigDecimal.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new BigDecimal(str);
         }
@@ -195,6 +209,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * AtomicLong converter.
      */
     ATOMIC_LONG(AtomicLong.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             long val = Long.parseLong(str);
             return new AtomicLong(val);
@@ -204,6 +219,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * AtomicLong converter.
      */
     ATOMIC_INTEGER(AtomicInteger.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             int val = Integer.parseInt(str);
             return new AtomicInteger(val);
@@ -213,6 +229,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * AtomicBoolean converter.
      */
     ATOMIC_BOOLEAN(AtomicBoolean.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             if ("true".equalsIgnoreCase(str)) {
                 return new AtomicBoolean(true);
@@ -227,6 +244,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Locale converter.
      */
     LOCALE(Locale.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             String[] split = str.split("_", 3);
             switch (split.length) {
@@ -248,11 +266,12 @@ enum JDKStringConverter implements StringConverter<Object> {
         public String convertToString(Object object) {
             return ((Class<?>) object).getName();
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             try {
-                return getClass().getClassLoader().loadClass(str);
+                return RenameHandler.INSTANCE.lookupType(str);
             } catch (ClassNotFoundException ex) {
-                throw new RuntimeException("Unable to create class: " + str, ex);
+                throw new RuntimeException("Unable to create type: " + str, ex);
             }
         }
     },
@@ -264,6 +283,7 @@ enum JDKStringConverter implements StringConverter<Object> {
         public String convertToString(Object object) {
             return ((Package) object).getName();
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return Package.getPackage(str);
         }
@@ -272,6 +292,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Currency converter.
      */
     CURRENCY(Currency.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return Currency.getInstance(str);
         }
@@ -284,6 +305,7 @@ enum JDKStringConverter implements StringConverter<Object> {
         public String convertToString(Object object) {
             return ((TimeZone) object).getID();
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return TimeZone.getTimeZone(str);
         }
@@ -292,6 +314,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * UUID converter.
      */
     UUID(UUID.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return java.util.UUID.fromString(str);
         }
@@ -300,6 +323,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * URL converter.
      */
     URL(URL.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             try {
                 return new URL(str);
@@ -312,6 +336,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * URI converter.
      */
     URI(URI.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return java.net.URI.create(str);
         }
@@ -324,6 +349,7 @@ enum JDKStringConverter implements StringConverter<Object> {
         public String convertToString(Object object) {
             return ((InetAddress) object).getHostAddress();
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             try {
                 return InetAddress.getByName(str);
@@ -336,6 +362,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * File converter.
      */
     FILE(File.class) {
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             return new File(str);
         }
@@ -350,6 +377,7 @@ enum JDKStringConverter implements StringConverter<Object> {
             String str = f.format(object);
             return str.substring(0, 26) + ":" + str.substring(26);
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             if (str.length() != 29) {
                 throw new IllegalArgumentException("Unable to parse date: " + str);
@@ -378,6 +406,7 @@ enum JDKStringConverter implements StringConverter<Object> {
             String str = f.format(cal.getTime());
             return str.substring(0, 26) + ":" + str.substring(26) + "[" + cal.getTimeZone().getID() + "]";
         }
+        @Override
         public Object convertFromString(Class<?> cls, String str) {
             if (str.length() < 31 || str.charAt(26) != ':'
                     || str.charAt(29) != '[' || str.charAt(str.length() - 1) != ']') {
@@ -397,19 +426,6 @@ enum JDKStringConverter implements StringConverter<Object> {
             }
         }
     },
-    /**
-     * Enum converter.
-     */
-    ENUM(Enum.class) {
-        @SuppressWarnings("rawtypes")
-        public String convertToString(Object object) {
-            return ((Enum) object).name();  // avoid toString() as that can be overridden
-        }
-        @SuppressWarnings({ "unchecked", "rawtypes" })
-        public Object convertFromString(Class cls, String str) {
-            return Enum.valueOf(cls, str);
-        }
-    },
     ;
 
     /** The type. */
@@ -419,7 +435,7 @@ enum JDKStringConverter implements StringConverter<Object> {
      * Creates an enum.
      * @param type  the type, not null
      */
-    private JDKStringConverter(Class<?> type) {
+    JDKStringConverter(Class<?> type) {
         this.type = type;
     }
 
@@ -431,9 +447,86 @@ enum JDKStringConverter implements StringConverter<Object> {
         return type;
     }
 
+    /**
+     * Gets the type of the converter.
+     * @return the type, not null
+     */
+    @Override
+    public Class<?> getEffectiveType() {
+        return type;
+    }
+
     //-----------------------------------------------------------------------
+    @Override
     public String convertToString(Object object) {
         return object.toString();
     }
 
+    //-----------------------------------------------------------------------
+    private static String base64Str = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
+    private static char[] base64Array = base64Str.toCharArray();
+    private static final int MASK_8BIT = 0xff;
+    private static final int MASK_6BIT = 0x3f;
+
+    private static String printBase64Binary(byte[] array) {
+        int len = array.length;
+        char[] buf = new char[((len + 2) / 3) * 4];
+        int pos = 0;
+        for (int i = 0; i < len; i += 3) {
+            int remaining = len - i;
+            if (remaining >= 3) {
+                int bits = (array[i] & MASK_8BIT) << 16 | (array[i + 1] & MASK_8BIT) <<  8 | (array[i + 2] & MASK_8BIT);
+                buf[pos++] = base64Array[(bits >>> 18) & MASK_6BIT];
+                buf[pos++] = base64Array[(bits >>> 12) & MASK_6BIT];
+                buf[pos++] = base64Array[(bits >>> 6) & MASK_6BIT];
+                buf[pos++] = base64Array[bits & MASK_6BIT];
+            } else if (remaining == 2) {
+                int bits = (array[i] & MASK_8BIT) << 16 | (array[i + 1] & MASK_8BIT) <<  8;
+                buf[pos++] = base64Array[(bits >>> 18) & MASK_6BIT];
+                buf[pos++] = base64Array[(bits >>> 12) & MASK_6BIT];
+                buf[pos++] = base64Array[(bits >>> 6) & MASK_6BIT];
+                buf[pos++] = '=';
+            } else {
+                int bits = (array[i] & MASK_8BIT) << 16;
+                buf[pos++] = base64Array[(bits >>> 18) & MASK_6BIT];
+                buf[pos++] = base64Array[(bits >>> 12) & MASK_6BIT];
+                buf[pos++] = '=';
+                buf[pos++] = '=';
+            }
+        }
+        return new String(buf);
+    }
+
+    private static byte[] parseBase64Binary(String str) {
+        // strict parser, must have length divisble by 4
+        if (str.length() % 4 != 0) {
+            throw new IllegalArgumentException("Invalid Base64 string");
+        }
+        // base64Str has 65 characters, with '=' at the end which is masked away
+        int parsedLen = (str.length() * 3) / 4;
+        byte[] decoded = new byte[parsedLen];
+        char[] inChars = str.toCharArray();
+        int pos = 0;
+        for (int i = 0; i < inChars.length; ) {
+            int bits = (base64Str.indexOf(inChars[i++]) & MASK_6BIT) << 18 |
+                            (base64Str.indexOf(inChars[i++]) & MASK_6BIT) << 12 |
+                            (base64Str.indexOf(inChars[i++]) & MASK_6BIT) << 6 |
+                            (base64Str.indexOf(inChars[i++]) & MASK_6BIT);
+            decoded[pos++] = (byte) ((bits >>> 16) & MASK_8BIT);
+            decoded[pos++] = (byte) ((bits >>> 8) & MASK_8BIT);
+            decoded[pos++] = (byte)  (bits & MASK_8BIT);
+        }
+        // fixup avoiding Arrays.copyRange
+        if (str.endsWith("==")) {
+            byte[] result = new byte[parsedLen - 2];
+            System.arraycopy(decoded, 0, result, 0, parsedLen - 2);
+            return result;
+        } else if (str.endsWith("=")) {
+            byte[] result = new byte[parsedLen - 1];
+            System.arraycopy(decoded, 0, result, 0, parsedLen - 1);
+            return result;
+        }
+        return decoded;
+    }
+
 }
