@@ -50,7 +50,7 @@ public final class StringConvert {
     /**
      * The cached null object.
      */
-    private static final StringConverter<?> CACHED_NULL = new StringConverter<Object>() {
+    private static final TypedStringConverter<?> CACHED_NULL = new TypedStringConverter<Object>() {
         @Override
         public String convertToString(Object object) {
             return null;
@@ -59,6 +59,10 @@ public final class StringConvert {
         public Object convertFromString(Class<? extends Object> cls, String str) {
             return null;
         }
+        @Override
+        public Class<?> getEffectiveType() {
+            return null;
+        }
     };
 
     /**
@@ -68,7 +72,7 @@ public final class StringConvert {
     /**
      * The cache of converters.
      */
-    private final ConcurrentMap<Class<?>, StringConverter<?>> registered = new ConcurrentHashMap<Class<?>, StringConverter<?>>();
+    private final ConcurrentMap<Class<?>, TypedStringConverter<?>> registered = new ConcurrentHashMap<Class<?>, TypedStringConverter<?>>();
 
     //-----------------------------------------------------------------------
     /**
@@ -150,6 +154,8 @@ public final class StringConvert {
             registered.put(Float.TYPE, JDKStringConverter.FLOAT);
             registered.put(Double.TYPE, JDKStringConverter.DOUBLE);
             registered.put(Character.TYPE, JDKStringConverter.CHARACTER);
+            // Guava
+            tryRegisterGuava();
             // JDK 1.8 classes
             tryRegister("java.time.Instant", "parse");
             tryRegister("java.time.Duration", "parse");
@@ -202,6 +208,34 @@ public final class StringConvert {
             this.factories.addAll(Arrays.asList(factories));
         }
         this.factories.add(AnnotationStringConverterFactory.INSTANCE);
+        if (includeJdkConverters) {
+            this.factories.add(EnumStringConverterFactory.INSTANCE);
+        }
+    }
+
+    /**
+     * Tries to register the Guava converters class.
+     * 
+     * @param className  the class name, not null
+     */
+    private void tryRegisterGuava() {
+        try {
+            RenameHandler.INSTANCE.loadType("com.google.common.reflect.Types");
+            @SuppressWarnings("unchecked")
+            Class<?> cls = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
+                    .loadType("org.joda.convert.TypeTokenStringConverter");
+            TypedStringConverter<?> conv = (TypedStringConverter<?>) cls.newInstance();
+            registered.put(conv.getEffectiveType(), conv);
+
+            @SuppressWarnings("unchecked")
+            Class<?> cls2 = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
+                    .loadType("org.joda.convert.TypeStringConverter");
+            TypedStringConverter<?> conv2 = (TypedStringConverter<?>) cls2.newInstance();
+            registered.put(conv2.getEffectiveType(), conv2);
+
+        } catch (Throwable ex) {
+            // ignore
+        }
     }
 
     /**
@@ -211,9 +245,9 @@ public final class StringConvert {
      */
     private void tryRegister(String className, String fromStringMethodName) {
         try {
-            Class<?> cls = getClass().getClassLoader().loadClass(className);
+            Class<?> cls = RenameHandler.INSTANCE.lookupType(className);
             registerMethods(cls, "toString", fromStringMethodName);
-        } catch (Exception ex) {
+        } catch (Throwable ex) {
             // ignore
         }
     }
@@ -306,6 +340,7 @@ public final class StringConvert {
      * class hierarchy and immediate parent interfaces.
      * It then searches for {@code ToString} and {@code FromString} annotations on the
      * specified class, class hierarchy or immediate parent interfaces.
+     * Finally, it handles {@code Enum} instances.
      * 
      * @param <T>  the type of the converter
      * @param cls  the class to find a converter for, not null
@@ -313,11 +348,7 @@ public final class StringConvert {
      * @throws RuntimeException (or subclass) if no converter found
      */
     public <T> StringConverter<T> findConverter(final Class<T> cls) {
-        StringConverter<T> conv = findConverterQuiet(cls);
-        if (conv == null) {
-            throw new IllegalStateException("No registered converter found: " + cls);
-        }
-        return conv;
+        return findTypedConverter(cls);
     }
 
     /**
@@ -332,15 +363,79 @@ public final class StringConvert {
      * class hierarchy and immediate parent interfaces.
      * It then searches for {@code ToString} and {@code FromString} annotations on the
      * specified class, class hierarchy or immediate parent interfaces.
+     * Finally, it handles {@code Enum} instances.
      * 
      * @param cls  the class to find a converter for, not null
      * @return the converter, using {@code Object} to avoid generics, not null
      * @throws RuntimeException (or subclass) if no converter found
      * @since 1.5
      */
-    @SuppressWarnings("unchecked")
     public StringConverter<Object> findConverterNoGenerics(final Class<?> cls) {
-        StringConverter<Object> conv = (StringConverter<Object>) findConverterQuiet(cls);
+        return findTypedConverterNoGenerics(cls);
+    }
+
+    /**
+     * Finds a suitable converter for the type.
+     * <p>
+     * This returns an instance of {@code TypedStringConverter} for the specified class.
+     * This is designed for user code where the {@code Class} object generics is known.
+     * <p>
+     * The search algorithm first searches the registered converters in the
+     * class hierarchy and immediate parent interfaces.
+     * It then searches for {@code ToString} and {@code FromString} annotations on the
+     * specified class, class hierarchy or immediate parent interfaces.
+     * Finally, it handles {@code Enum} instances.
+     * <p>
+     * The returned converter may be queried for the effective type of the conversion.
+     * This can be used to find the best type to send in a serialized form.
+     * <p>
+     * NOTE: Changing the method return type of {@link #findConverter(Class)}
+     * would be source compatible but not binary compatible. As this is a low-level
+     * library, binary compatibility is important, hence the addition of this method.
+     * 
+     * @param <T>  the type of the converter
+     * @param cls  the class to find a converter for, not null
+     * @return the converter, not null
+     * @throws RuntimeException (or subclass) if no converter found
+     * @since 1.7
+     */
+    public <T> TypedStringConverter<T> findTypedConverter(final Class<T> cls) {
+        TypedStringConverter<T> conv = findConverterQuiet(cls);
+        if (conv == null) {
+            throw new IllegalStateException("No registered converter found: " + cls);
+        }
+        return conv;
+    }
+
+    /**
+     * Finds a suitable converter for the type with open generics.
+     * <p>
+     * This returns an instance of {@code TypedStringConverter} for the specified class.
+     * This is designed for framework usage where the {@code Class} object generics are unknown'?'.
+     * The returned type is declared with {@code Object} instead of '?' to
+     * allow the {@link ToStringConverter} to be invoked.
+     * <p>
+     * The search algorithm first searches the registered converters in the
+     * class hierarchy and immediate parent interfaces.
+     * It then searches for {@code ToString} and {@code FromString} annotations on the
+     * specified class, class hierarchy or immediate parent interfaces.
+     * Finally, it handles {@code Enum} instances.
+     * <p>
+     * The returned converter may be queried for the effective type of the conversion.
+     * This can be used to find the best type to send in a serialized form.
+     * <p>
+     * NOTE: Changing the method return type of {@link #findConverterNoGenerics(Class)}
+     * would be source compatible but not binary compatible. As this is a low-level
+     * library, binary compatibility is important, hence the addition of this method.
+     * 
+     * @param cls  the class to find a converter for, not null
+     * @return the converter, using {@code Object} to avoid generics, not null
+     * @throws RuntimeException (or subclass) if no converter found
+     * @since 1.7
+     */
+    @SuppressWarnings("unchecked")
+    public TypedStringConverter<Object> findTypedConverterNoGenerics(final Class<?> cls) {
+        TypedStringConverter<Object> conv = (TypedStringConverter<Object>) findConverterQuiet(cls);
         if (conv == null) {
             throw new IllegalStateException("No registered converter found: " + cls);
         }
@@ -356,11 +451,11 @@ public final class StringConvert {
      * @throws RuntimeException if invalid
      */
     @SuppressWarnings("unchecked")
-    private <T> StringConverter<T> findConverterQuiet(final Class<T> cls) {
+    private <T> TypedStringConverter<T> findConverterQuiet(final Class<T> cls) {
         if (cls == null) {
             throw new IllegalArgumentException("Class must not be null");
         }
-        StringConverter<T> conv = (StringConverter<T>) registered.get(cls);
+        TypedStringConverter<T> conv = (TypedStringConverter<T>) registered.get(cls);
         if (conv == CACHED_NULL) {
             return null;
         }
@@ -389,12 +484,12 @@ public final class StringConvert {
      * @throws RuntimeException if invalid
      */
     @SuppressWarnings("unchecked")
-    private <T> StringConverter<T> findAnyConverter(final Class<T> cls) {
-        StringConverter<T> conv = null;
+    private <T> TypedStringConverter<T> findAnyConverter(final Class<T> cls) {
+        TypedStringConverter<T> conv = null;
         // check for registered on superclass
         Class<?> loopCls = cls.getSuperclass();
         while (loopCls != null && conv == null) {
-            conv = (StringConverter<T>) registered.get(loopCls);
+            conv = (TypedStringConverter<T>) registered.get(loopCls);
             if (conv != null && conv != CACHED_NULL) {
                 return conv;
             }
@@ -402,16 +497,16 @@ public final class StringConvert {
         }
         // check for registered on interfaces
         for (Class<?> loopIfc : cls.getInterfaces()) {
-            conv = (StringConverter<T>) registered.get(loopIfc);
+            conv = (TypedStringConverter<T>) registered.get(loopIfc);
             if (conv != null && conv != CACHED_NULL) {
                 return conv;
             }
         }
         // check factories
         for (StringConverterFactory factory : factories) {
-            conv = (StringConverter<T>) factory.findConverter(cls);
-            if (conv != null) {
-                return conv;
+            StringConverter<T> factoryConv = (StringConverter<T>) factory.findConverter(cls);
+            if (factoryConv != null) {
+                return TypedAdapter.adapt(cls, factoryConv);
             }
         }
         return null;
@@ -463,7 +558,7 @@ public final class StringConvert {
         if (this == INSTANCE) {
             throw new IllegalStateException("Global singleton cannot be extended");
         }
-        registered.put(cls, converter);
+        registered.put(cls, TypedAdapter.adapt(cls, converter));
     }
 
     /**
@@ -490,13 +585,19 @@ public final class StringConvert {
         if (fromString == null || toString == null) {
             throw new IllegalArgumentException("Converters must not be null");
         }
-        register(cls, new StringConverter<T>() {
+        register(cls, new TypedStringConverter<T>() {
+            @Override
             public String convertToString(T object) {
                 return toString.convertToString(object);
             }
+            @Override
             public T convertFromString(Class<? extends T> cls, String str) {
                 return fromString.convertFromString(cls, str);
             }
+            @Override
+            public Class<?> getEffectiveType() {
+                return cls;
+            }
         });
     }
 
@@ -531,7 +632,7 @@ public final class StringConvert {
         }
         Method toString = findToStringMethod(cls, toStringMethodName);
         Method fromString = findFromStringMethod(cls, fromStringMethodName);
-        MethodsStringConverter<T> converter = new MethodsStringConverter<T>(cls, toString, fromString);
+        MethodsStringConverter<T> converter = new MethodsStringConverter<T>(cls, toString, fromString, cls);
         registered.putIfAbsent(cls, converter);
     }
 
