@@ -17,6 +17,7 @@ package org.joda.convert;
 
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import java.io.File;
 import java.math.BigDecimal;
@@ -144,6 +145,26 @@ public class TestJDKStringConverters {
         assertTrue(Arrays.equals(array, (byte[]) test.convertFromString(byte[].class, str)));
     }
 
+    @Test
+    public void test_byteArray3() {
+        JDKStringConverter test = JDKStringConverter.BYTE_ARRAY;
+        byte[] array = new byte[] {77};
+        String str = "TQ==";
+        assertEquals(byte[].class, test.getType());
+        assertEquals(str, test.convertToString(array));
+        assertTrue(Arrays.equals(array, (byte[]) test.convertFromString(byte[].class, str)));
+    }
+
+    @Test
+    public void test_byteArray4() {
+        JDKStringConverter test = JDKStringConverter.BYTE_ARRAY;
+        byte[] array = new byte[] {73, 97, 112, 77};
+        String str = "SWFwTQ==";
+        assertEquals(byte[].class, test.getType());
+        assertEquals(str, test.convertToString(array));
+        assertTrue(Arrays.equals(array, (byte[]) test.convertFromString(byte[].class, str)));
+    }
+
     @Test
     public void test_Boolean() {
         JDKStringConverter test = JDKStringConverter.BOOLEAN;
@@ -245,6 +266,18 @@ public class TestJDKStringConverters {
         JDKStringConverter.CLASS.convertFromString(Class.class, "RUBBISH");
     }
 
+    @Test
+    public void test_Class_withRename() {
+        try {
+            JDKStringConverter.CLASS.convertFromString(Class.class, "org.foo.StringConvert");
+            fail();
+        } catch (RuntimeException ex) {
+            // expected
+        }
+        RenameHandler.INSTANCE.renamedType("org.foo.StringConvert", StringConvert.class);
+        assertEquals(StringConvert.class, JDKStringConverter.CLASS.convertFromString(Class.class, "org.foo.StringConvert"));
+    }
+
     @Test
     public void test_Package() {
         JDKStringConverter test = JDKStringConverter.PACKAGE;
@@ -298,7 +331,13 @@ public class TestJDKStringConverters {
     public void test_InetAddress() throws Exception {
         JDKStringConverter test = JDKStringConverter.INET_ADDRESS;
         doTest(test, InetAddress.class, InetAddress.getByName("1.2.3.4"), "1.2.3.4");
-        doTest(test, InetAddress.class, InetAddress.getByName("2001:0db8:85a3:0000:0000:8a2e:0370:7334"), "2001:db8:85a3:0:0:8a2e:370:7334");
+        
+        InetAddress obj = InetAddress.getByName("2001:0db8:85a3:0000:0000:8a2e:0370:7334");
+        String str = test.convertToString(obj);
+        assertTrue(str.equals("2001:db8:85a3:0:0:8a2e:370:7334") || str.equals("2001:db8:85a3::8a2e:370:7334"));
+        assertEquals(obj, test.convertFromString(InetAddress.class, str));
+        assertEquals(obj, test.convertFromString(InetAddress.class, "2001:db8:85a3:0:0:8a2e:370:7334"));
+        assertEquals(obj, test.convertFromString(InetAddress.class, "2001:db8:85a3::8a2e:370:7334"));
     }
 
 //    @Test(expected=RuntimeException.class)
@@ -398,15 +437,35 @@ public class TestJDKStringConverters {
 
     @Test
     public void test_Enum() {
-        JDKStringConverter test = JDKStringConverter.ENUM;
-        assertEquals(Enum.class, test.getType());
+        TypedStringConverter<RoundingMode> test = StringConvert.create().findTypedConverter(RoundingMode.class);
+        assertEquals(RoundingMode.class, test.getEffectiveType());
         assertEquals("CEILING", test.convertToString(RoundingMode.CEILING));
         assertEquals(RoundingMode.CEILING, test.convertFromString(RoundingMode.class, "CEILING"));
     }
 
     @Test(expected=RuntimeException.class)
     public void test_Enum_invalidConstant() {
-        JDKStringConverter.ENUM.convertFromString(RoundingMode.class, "RUBBISH");
+        TypedStringConverter<RoundingMode> test = StringConvert.create().findTypedConverter(RoundingMode.class);
+        test.convertFromString(RoundingMode.class, "RUBBISH");
+    }
+
+    @Test
+    public void test_Enum_withRename() {
+        TypedStringConverter<Status> test = StringConvert.create().findTypedConverter(Status.class);
+        assertEquals("VALID", test.convertToString(Status.VALID));
+        assertEquals("INVALID", test.convertToString(Status.INVALID));
+        assertEquals(Status.VALID, test.convertFromString(Status.class, "VALID"));
+        assertEquals(Status.INVALID, test.convertFromString(Status.class, "INVALID"));
+        try {
+            test.convertFromString(Status.class, "OK");
+            fail();
+        } catch (RuntimeException ex) {
+            // expected
+        }
+        RenameHandler.INSTANCE.renamedEnum("OK", Status.VALID);
+        assertEquals(Status.VALID, test.convertFromString(Status.class, "OK"));
+        assertEquals(Status.VALID, test.convertFromString(Status.class, "VALID"));
+        assertEquals(Status.INVALID, test.convertFromString(Status.class, "INVALID"));
     }
 
     //-----------------------------------------------------------------------
